@@ -56,11 +56,12 @@
 #include "logging.h"
 #include "tpm_ioctl.h"
 #include "tpmlib.h"
-#include "swtpm_nvfile.h"
+#include "swtpm_nvstore.h"
 #include "locality.h"
 #include "mainloop.h"
 #include "utils.h"
 #include "swtpm_debug.h"
+#include "swtpm_utils.h"
 
 /* local variables */
 
@@ -790,8 +791,7 @@ int ctrlchannel_process_fd(int fd,
             goto err_bad_input;
         }
 
-        mlp->flags = MAIN_LOOP_FLAG_USE_FD | MAIN_LOOP_FLAG_KEEP_CONNECTION |
-                       MAIN_LOOP_FLAG_END_ON_HUP;
+        mlp->flags |= MAIN_LOOP_FLAG_USE_FD | MAIN_LOOP_FLAG_KEEP_CONNECTION;
         mlp->fd = *data_fd;
 
         *res_p = htobe32(TPM_SUCCESS);
