@@ -57,7 +57,7 @@
 #include "main.h"
 #include "swtpm_debug.h"
 #include "swtpm_io.h"
-#include "swtpm_nvfile.h"
+#include "swtpm_nvstore.h"
 #include "common.h"
 #include "locality.h"
 #include "logging.h"
@@ -70,7 +70,7 @@
 #include "vtpm_proxy.h"
 #endif
 #include "tpmstate.h"
-#include "osx.h"
+#include "daemonize.h"
 #include "seccomp_profile.h"
 #include "options.h"
 #include "capabilities.h"
@@ -175,10 +175,10 @@ static void usage(FILE *file, const char *prgname, const char *iface)
     "                   derived from this passphrase; default kdf is PBKDF2\n"
     "--pid file=<path>|fd=<filedescriptor>\n"
     "                 : write the process ID into the given file\n"
-    "--tpmstate dir=<dir>[,mode=0...]\n"
-    "                 : set the directory where the TPM's state will be written\n"
+    "--tpmstate dir=<dir>[,mode=0...]|backend-uri=<uri>\n"
+    "                 : set the directory or uri where the TPM's state will be written\n"
     "                   into; the TPM_PATH environment variable can be used\n"
-    "                   instead;\n"
+    "                   instead of dir option;\n"
     "                   mode allows a user to set the file mode bits of the state files;\n"
     "                   the default mode is 0640;\n"
     "-r|--runas <user>: change to the given user\n"
@@ -207,6 +207,8 @@ static void usage(FILE *file, const char *prgname, const char *iface)
 #endif
     "--print-capabilites\n"
     "                 : print capabilities and terminate\n"
+    "--print-states\n"
+    "                 : print existing TPM states and terminate\n"
     "-h|--help        : display this help screen and terminate\n"
     "\n",
     prgname, iface);
@@ -294,6 +296,7 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
     bool need_init_cmd = true;
     unsigned int seccomp_action;
     bool printcapabilities = false;
+    bool printstates = false;
     static struct option longopts[] = {
         {"daemon"    ,       no_argument, 0, 'd'},
         {"help"      ,       no_argument, 0, 'h'},
@@ -317,6 +320,7 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
 #endif
         {"print-capabilities"
                      ,       no_argument, 0, 'a'},
+        {"print-states",     no_argument, 0, 'e'},
         {NULL        , 0                , 0, 0  },
     };
 
@@ -331,6 +335,11 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
         switch (opt) {
         case 'd':
             daemonize = TRUE;
+            if (daemonize_prep() == -1) {
+                logprintf(STDERR_FILENO,
+                          "Could not prepare to daemonize: %s\n", strerror(errno));
+                exit(EXIT_FAILURE);
+            }
             break;
 
         case 'c':
@@ -423,6 +432,10 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
             printcapabilities = true;
             break;
 
+        case 'e':
+            printstates = true;
+            break;
+
         case 'r':
             runas = optarg;
             break;
@@ -449,6 +462,15 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
         exit(EXIT_FAILURE);
     }
 
+    /* change process ownership before accessing files */
+    if (runas) {
+        if (change_process_owner(runas) < 0)
+            exit(EXIT_FAILURE);
+    }
+
+    if (handle_log_options(logdata) < 0)
+        exit(EXIT_FAILURE);
+
     if (handle_locality_options(localitydata, &mlp.locality_flags) < 0)
         exit(EXIT_FAILURE);
 
@@ -459,22 +481,38 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
     }
 #endif
 
-    /*
-     * choose the TPM version early so that getting/setting
-     * buffer size works.
-     */
+    if (printcapabilities) {
+        /*
+         * Choose the TPM version so that getting/setting buffer size works.
+         * Ignore failure, for backward compatibility when TPM 1.2 is disabled.
+         */
+        TPMLIB_ChooseTPMVersion(mlp.tpmversion);
+        ret = capabilities_print_json(false);
+        exit(ret ? EXIT_FAILURE : EXIT_SUCCESS);
+    }
+
     if (TPMLIB_ChooseTPMVersion(mlp.tpmversion) != TPM_SUCCESS) {
         logprintf(STDERR_FILENO,
                   "Error: Could not choose TPM version.\n");
         exit(EXIT_FAILURE);
     }
 
-    if (printcapabilities) {
-        ret = capabilities_print_json(false);
-        exit(ret ? EXIT_FAILURE : EXIT_SUCCESS);
-    }
+    tpmstate_set_version(mlp.tpmversion);
 
-    SWTPM_NVRAM_Set_TPMVersion(mlp.tpmversion);
+    if (printstates) {
+        if (handle_tpmstate_options(tpmstatedata) < 0)
+            exit(EXIT_FAILURE);
+        if (tpmstatedata == NULL) {
+            logprintf(STDERR_FILENO,
+                      "Error: --tpmstate option is required for --print-states\n");
+            exit(EXIT_FAILURE);
+        }
+        ret = SWTPM_NVRAM_PrintJson();
+        if (ret)
+            goto exit_failure;
+        else
+            goto exit_success;
+    }
 
     if (mlp.fd < 0) {
         logprintf(STDERR_FILENO,
@@ -491,14 +529,7 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
         goto exit_failure;
     }
 
-    /* change process ownership before accessing files */
-    if (runas) {
-        if (change_process_owner(runas) < 0)
-            goto exit_failure;
-    }
-
-    if (handle_log_options(logdata) < 0 ||
-        handle_key_options(keydata) < 0 ||
+    if (handle_key_options(keydata) < 0 ||
         handle_migration_key_options(migkeydata) < 0 ||
         handle_pid_options(piddata) < 0 ||
         handle_tpmstate_options(tpmstatedata) < 0 ||
@@ -508,17 +539,6 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
         goto exit_failure;
     }
 
-    if (daemonize) {
-#if defined __APPLE__
-       if (0 != osx_daemon(1, 0)) {
-#else
-       if (0 != daemon(1, 0)) {
-#endif
-           logprintf(STDERR_FILENO, "Error: Could not daemonize.\n");
-           goto exit_failure;
-       }
-    }
-
     if (pidfile_write(getpid()) < 0) {
         goto exit_failure;
     }
@@ -552,6 +572,10 @@ int swtpm_chardev_main(int argc, char **argv, const char *prgname, const char *i
     mlp.flags |= MAIN_LOOP_FLAG_USE_FD | MAIN_LOOP_FLAG_KEEP_CONNECTION | \
       MAIN_LOOP_FLAG_END_ON_HUP;
 
+    if (daemonize) {
+        daemonize_finish();
+    }
+
     rc = mainLoop(&mlp, notify_fd[0]);
 
 error_seccomp_profile:
@@ -581,4 +605,9 @@ exit_failure:
     swtpm_cleanup(mlp.cc);
 
     exit(EXIT_FAILURE);
+
+exit_success:
+    swtpm_cleanup(mlp.cc);
+
+    exit(EXIT_SUCCESS);
 }
