@@ -61,6 +61,7 @@
 #include "sys_dependencies.h"
 #include "tpm_asn1.h"
 #include "swtpm.h"
+#include "compiler_dependencies.h"
 
 enum cert_type_t {
     CERT_TYPE_EK = 1,
@@ -92,7 +93,7 @@ typedef struct TCG_PCCLIENT_STORED_FULL_CERT_HEADER {
 #define TCG_TAG_PCCLIENT_FULL_CERT 0x1002
 
 static void
-versioninfo(const char *prg)
+versioninfo()
 {
     fprintf(stdout,
         "TPM certificate tool version %d.%d.%d, Copyright (c) 2015 IBM Corp.\n"
@@ -102,7 +103,7 @@ versioninfo(const char *prg)
 static void
 usage(const char *prg)
 {
-    versioninfo(prg);
+    versioninfo();
     fprintf(stdout,
         "\nUsage: %s [options]\n"
         "\n"
@@ -125,7 +126,8 @@ usage(const char *prg)
         "--ecc-curveid <id>        : ECC curve id; secp256r1, secp384r1, secp521r1\n"
         "                            default: secp256r1\n"
         "--serial <serial number>  : The certificate serial number\n"
-        "--days <number>           : Number of days the cert is valid\n"
+        "--days <number>           : Number of days the cert is valid;\n"
+        "                            -1 for no expiration\n"
         "--pem                     : Write certificate in PEM format; default is DER\n"
         "--type <platform|ek>      : The type of certificate to create; default is ek\n"
         "--tpm-manufacturer <name> : The name of the TPM manufacturer\n"
@@ -894,9 +896,11 @@ exit:
     return err;
 }
 
-static int mypinfunc(void *userdata, int attempt, const char *tokenurl,
-                     const char *token_label,
-                     unsigned int flags,
+static int mypinfunc(void *userdata SWTPM_ATTR_UNUSED,
+                     int attempt SWTPM_ATTR_UNUSED,
+                     const char *tokenurl SWTPM_ATTR_UNUSED,
+                     const char *token_label SWTPM_ATTR_UNUSED,
+                     unsigned int flags SWTPM_ATTR_UNUSED,
                      char *pin, size_t pin_max)
 {
     const char *userpin = getenv("SWTPM_PKCS11_PIN");
@@ -982,8 +986,9 @@ static void capabilities_print_json()
             "\"features\": [ "
              "\"cmdarg-signkey-pwd\""
              ", \"cmdarg-parentkey-pwd\""
-              " ] "
-             "}\n");
+            " ], "
+            "\"version\": \"" VERSION "\" "
+            "}\n");
 }
 
 int
@@ -1015,6 +1020,7 @@ main(int argc, char *argv[])
     const char *subject = NULL;
     const char *error = NULL;
     int days = 365;
+    time_t exp_time;
     char *sigkeypass = NULL;
     char *parentkeypass = NULL;
     uint64_t ser_number;
@@ -1160,10 +1166,6 @@ main(int argc, char *argv[])
             break;
         case 'd': /* --days */
             days = atoi(optarg);
-            if (days < 0) {
-                fprintf(stderr, "Days must be a positive number.\n");
-                goto cleanup;
-            }
             break;
         case 'r': /* --serial */
             errno = 0;
@@ -1238,7 +1240,7 @@ main(int argc, char *argv[])
             capabilities_print_json();
             exit(0);
         case 'v': /* --version */
-            versioninfo(argv[0]);
+            versioninfo();
             exit(0);
         case 'h': /* --help */
             usage(argv[0]);
@@ -1449,8 +1451,8 @@ if (_err != GNUTLS_E_SUCCESS) {             \
     CHECK_GNUTLS_ERROR(err, "Could not set activation time on CRT: %s\n",
                        gnutls_strerror(err))
 
-    err = gnutls_x509_crt_set_expiration_time(crt,
-             now + (time_t)days * 24 * 60 * 60);
+    exp_time = (days < 0) ? -1 : now + (time_t)days * 24 * 60 * 60;
+    err = gnutls_x509_crt_set_expiration_time(crt, exp_time);
     CHECK_GNUTLS_ERROR(err, "Could not set expiration time on CRT: %s\n",
                        gnutls_strerror(err))
 
