@@ -420,8 +420,6 @@ static int do_save_state_blob(int fd, bool is_chardev, const char *blobtype,
         } else {
             offset += devtoh32(is_chardev, pgs.u.resp.length);
         }
-        if (!is_chardev)
-            break;
     }
 
     close(file_fd);
@@ -858,7 +856,7 @@ static int parse_tcp_optarg(char *optarg, char **tcp_hostname, int *tcp_port)
     return 0;
 }
 
-static void versioninfo(const char *prgname)
+static void versioninfo(void)
 {
     fprintf(stdout,
 "TPM emulator control tool version %d.%d.%d, Copyright (c) 2015 IBM Corp.\n"
@@ -867,7 +865,7 @@ static void versioninfo(const char *prgname)
 
 static void usage(const char *prgname)
 {
-    versioninfo(prgname);
+    versioninfo();
     fprintf(stdout,
 "\n"
 "Usage: %s command <device path>\n"
@@ -1044,7 +1042,7 @@ int main(int argc, char *argv[])
             }
             break;
         case 'V':
-            versioninfo(argv[0]);
+            versioninfo();
             ret = EXIT_SUCCESS;
             goto exit;
         case 'H':
